/*                                                                          
  Ϳ
   Ints.h  (versin 0.2)                                                  
                                                                          
   Librera con algunas funciones ASM usando interrupciones.              
   Estas funciones usan una mnima parte de los servicios que nos propor- 
   ciona la int 10h. Se han eliminado Int_PutPixel() e Int_GetPixel() de- 
   bido a que ahora disponemos de funciones assembler mucho ms rpidas.  
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/

/*  Funciones implementadas:  */

/* INTERRUPCIONES SOFTWARE */

void SetVideoMode( char );               /* inicializar modos de video */
char GetVideoMode( void );               /* leer el modo de video actual */
void SetBorder( char );                  /* Cambiar el color del borde */
void WaitVRetraceC(void);                /* Se espera al retrazo vertical */
void WaitVRetrace(void);                 /* se espera al retrazo vertical */


/*                                                                          
  Ŀ
   SetVideoMode( char modo );                                             
                                                                          
   Inicializa el modo de video especificado en el parmetro <modo>, sien- 
   do ste un nmero entre 0 y 0x13 (Ver modos de video en la revista).   
   Utiliza el servicio 0 (Set VMode) de la int 10h.                       
  
*/

void SetVideoMode( char modo )
{
asm {
        xor ah, ah                       /* AH = 0 (Servicio SetVideoMode) */
        mov al, [modo]                   /* AL = Modo de video */
        int 10h                          /* llamada a la interrupcin 10h */
    }
}


/*                                                                          
  Ŀ
   modo = GetVideoMode( void );                                           
                                                                          
   Devuelve el modo de video actual. Utiliza int 10h, servicio 0Fh.       
  
*/

char GetVideoMode( void )
{
char modo;
asm {
        mov ah, 0x0F                     /* AH = 0Fh (GetVideoMode) */
        int 10h                          /* llamada a la interrupcin 10h */
        mov [modo], al                   /* AL = Modo de video */
    }
 return( modo );                         /* devolver el valor leido */
}


/*                                                                          
  Ŀ
   SetBorder( char color );                                               
                                                                          
   Modifica el color del borde mediante el servicio 1001h de la int 10h.  
   No es muy til pero es otro ejemplo de uso de interrupciones.          
  
*/

void SetBorder( char color )
{
asm {
        mov ax, 0x1001                   /* AX = 01001h (SetBorderColour) */
        mov bh, [color]                  /* BH = Color del borde */
        int 10h                          /* llamada a la interrupcin 10h */
    }
}

/*                                                                          
  Ŀ
   WaitVRetraceC ( void );                                                
                                                                          
   Se espera al retrazo vertical mediante rdenes del C.                  
  
*/

void WaitVRetraceC(void)
{
  while ((inportb(0x03DA) & 8) != 0);
  while ((inportb(0x03DA) & 8) == 0);
}

/*                                                                          
  Ŀ
   WaitVRetrace  ( void );                                                
                                                                          
   Se espera al retrazo vertical mediante rdenes el ensamblador.         
   Aunque la funcin anterior hace la misma funcin, esta versin es ms  
   aconsejable por la rapidez del ensamblador, ya que el retrazo de la    
   pantalla tambin es muy rpido.                                        
  
*/

void WaitVRetrace (void)
{
asm   mov dx, 0x03DA
Retrazo1:
asm {
        in      al, dx
        test    al, 8
        jnz     Retrazo1
    }
Retrazo2:
asm {
        in      al, dx
        test    al, 8
        jz      Retrazo2
    }
}

